/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/indent/src/gf.h                                        *
 * Created:       2005-08-07 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2005-08-09 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2005 Hampa Hug <hampa@hampa.ch>                        *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: gf.h 40 2005-12-28 21:36:26Z hampa $ */


#ifndef CBASE_GF_H
#define CBASE_GF_H 1


#define GF2_POLY  0x00003  /* x^2 + x + 1 */
#define GF3_POLY  0x0000b  /* x^3 + x + 1 */
#define GF4_POLY  0x00013  /* x^4 + x + 1 */
#define GF5_POLY  0x00025  /* x^5 + x^2 + 1 */
#define GF6_POLY  0x00043  /* x^6 + x + 1 */
#define GF7_POLY  0x00089  /* x^7 + x^3 + 1 */
#define GF8_POLY  0x0011d  /* x^8 + x^4 + x^3 + x^2 + 1 */
#define GF9_POLY  0x00211  /* x^9 + x^4 + 1 */
#define GF10_POLY 0x00409  /* x^10 + x^3 + 1 */
#define GF11_POLY 0x00805  /* x^11 + x^2 + 1 */
#define GF12_POLY 0x01053  /* x^12 + x^6 + x^4 + x + 1 */
#define GF13_POLY 0x0201b  /* x^13 + x^4 + x^3 + x + 1 */
#define GF14_POLY 0x04443  /* x^14 + x^10 + x^6 + x + 1 */
#define GF15_POLY 0x08003  /* x^15 + x + 1 */
#define GF16_POLY 0x1100b  /* x^16 + x^12 + x^3 + x + 1 */


/*!***************************************************************************
 * @short The galois field type
 *****************************************************************************/
typedef struct {
	unsigned short k;
	unsigned long  poly;

	unsigned short high;
	unsigned short mask;

	unsigned short *exp;
	unsigned short *log;
} gf_t;


/*!***************************************************************************
 * @short  Initialize a galois field
 * @param  gf The galois field
 * @param  k  The field size GF(2^k), 2 <= k <= 16.
 * @return Zero if successful
 *****************************************************************************/
int gf_init (gf_t *gf, unsigned k);

/*!***************************************************************************
 * @short  Create a new galois field
 * @param  k The field size GF(2^k)
 * @return The new galois field
 *****************************************************************************/
gf_t *gf_new (unsigned k);

/*!***************************************************************************
 * @short Free a galois field
 * @param gf The galois field
 *****************************************************************************/
void gf_free (gf_t *gf);

/*!***************************************************************************
 * @short Delete a galois field
 * @param gf The galois field
 *****************************************************************************/
void gf_del (gf_t *gf);

unsigned short gf_add (const gf_t *gf, unsigned short x, unsigned short y);
unsigned short gf_sub (const gf_t *gf, unsigned short x, unsigned short y);
unsigned short gf_inv (const gf_t *gf, unsigned short x);
unsigned short gf_div (const gf_t *gf, unsigned short x, unsigned short y);


static inline
unsigned short gf_mul (const gf_t *gf, unsigned short x, unsigned short y)
{
	unsigned i;

	if ((x == 0) || (y == 0)) {
		return (0);
	}

	x = gf->log[x];
	y = gf->log[y];

	/* use of 65535 instead of gf->mask as an optimization for GF(2^16) */
	if ((65535 - y) <= x) {
		i = x + y - 65535;
	}
	else {
		i = x + y;
	}

	return (gf->exp[i]);
}


#endif
