/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/md5.h                                                  *
 * Created:       2001-06-04 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2005-08-08 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2001-2005 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: md5.h 40 2005-12-28 21:36:26Z hampa $ */


#ifndef LECO_MD5_H
#define LECO_MD5_H 1


/*!***************************************************************************
 * @short The MD5 context
 *****************************************************************************/
typedef struct {
	unsigned long A;
	unsigned long B;
	unsigned long C;
	unsigned long D;

	unsigned long cnt[2];

	unsigned char buf[64];
	unsigned      buf_i;
} md5_t;



/*!***************************************************************************
 * @short Initialize an MD5 context
 *****************************************************************************/
void md5_init (md5_t *md5);

/*!***************************************************************************
 * @short Copy an MD5 context
 *****************************************************************************/
void md5_copy (md5_t *dst, const md5_t *src);

/*!***************************************************************************
 * @short Calculate an MD5 sum
 * @param md5 The MD5 context
 * @param buf The data
 * @param cnt The number of bytes at buf
 *
 * This function can be called multiple times to add more bytes.
 *****************************************************************************/
void md5_calc (md5_t *md5, const void *buf, unsigned long cnt);

/*!***************************************************************************
 * @short End calculating an MD5 sum
 *
 * This function must be called after the last call to md5_calc() and
 * before getting the md5 sum.
 *****************************************************************************/
void md5_done (md5_t *md5);

/*!***************************************************************************
 * @short Get a 32bit word of the MD5 sum
 * @param md5 The MD5 context
 * @param i   The word index. 0 <= i <= 3.
 *****************************************************************************/
unsigned long md5_get_sum_i (md5_t *md5, unsigned i);

/*!***************************************************************************
 * @short Copy the MD5 sum to a byte buffer
 * @param md5 The MD5 context
 * @param buf The 16 byte MD5 buffer
 *****************************************************************************/
void md5_get_sum (md5_t *md5, void *buf);

/*!***************************************************************************
 * @short Calculate the MD5 sum of a buffer
 * @param dst The 16 byte MD5 buffer
 * @param src The source buffer
 * @param n   The size in bytes of the source buffer
 *****************************************************************************/
void md5_calc1 (void *dst, const void *src, unsigned long n);


#endif
